"""Modbus api for stiebel eltron heat pumps. This file is generated. Do not modify it manually."""

from enum import Enum

from . import (
    ENERGY_DATA_BLOCK_NAME,
    ENERGY_MANAGEMENT_SETTINGS_REGISTERS,
    ENERGY_SYSTEM_INFORMATION_REGISTERS,
    VIRTUAL_REGISTER_OFFSET,
    VIRTUAL_TOTAL_AND_DAY_REGISTER_OFFSET,
    IsgRegisters,
    ModbusRegister,
    ModbusRegisterBlock,
    RegisterType,
    StiebelEltronAPI,
)


class LwzSystemValuesRegisters(IsgRegisters):
    ACTUAL_ROOM_T_HC1 = 1
    SET_ROOM_TEMPERATURE_HC1 = 2
    RELATIVE_HUMIDITY_HC1 = 3
    ACTUAL_ROOM_T_HC2 = 4
    SET_ROOM_TEMPERATURE_HC2 = 5
    RELATIVE_HUMIDITY_HC2 = 6
    OUTSIDE_TEMPERATURE = 7
    ACTUAL_VALUE_HC1 = 8
    SET_VALUE_HC1 = 9
    ACTUAL_VALUE_HC2 = 10
    SET_VALUE_HC2 = 11
    FLOW_TEMPERATURE = 12
    RETURN_TEMPERATURE = 13
    PRESSURE_HTG_CIRC = 14
    FLOW_RATE = 15
    ACTUAL_DHW_T = 16
    DHW_SET_TEMPERATURE = 17
    VENTILATION_AIR_ACTUAL_FAN_SPEED = 18
    VENTILATION_AIR_SET_FLOW_RATE = 19
    EXTRACT_AIR_ACTUAL_FAN_SPEED = 20
    EXTRACT_AIR_SET_FLOW_RATE = 21
    EXTRACT_AIR_HUMIDITY = 22
    EXTRACT_AIR_TEMP = 23
    EXTRACT_AIR_DEW_POINT = 24
    DEW_POINT_TEMP_HC1 = 25
    DEW_POINT_TEMP_HC2 = 26
    COLLECTOR_TEMPERATURE = 27
    HOT_GAS_TEMPERATURE = 28
    HIGH_PRESSURE = 29
    LOW_PRESSURE = 30
    COMPRESSOR_STARTS_HI = 31
    COMPRESSOR_SPEED = 32
    MIXED_WATER_AMOUNT = 33
    COMPRESSOR_STARTS_LOW = 34
    COMPRESSOR_STARTS = 100032


class LwzSystemParametersRegisters(IsgRegisters):
    OPERATING_MODE = 1001
    ROOM_TEMPERATURE_DAY_HK1 = 1002
    ROOM_TEMPERATURE_DAY_AND_TOTAL_HK1 = 201003
    ROOM_TEMPERATURE_NIGHT_HK1 = 1003
    MANUAL_HC_SET_HK1 = 1004
    ROOM_TEMPERATURE_DAY_HK2 = 1005
    ROOM_TEMPERATURE_DAY_AND_TOTAL_HK2 = 201006
    ROOM_TEMPERATURE_NIGHT_HK2 = 1006
    MANUAL_HC_SET_HK2 = 1007
    GRADIENT_HK1 = 1008
    LOW_END_HK1 = 1009
    GRADIENT_HK2 = 1010
    LOW_END_HK2 = 1011
    DHW_SET_DAY = 1012
    DHW_SET_DAY_AND_TOTAL = 201013
    DHW_SET_NIGHT = 1013
    DHW_SET_MANUAL = 1014
    MWM_SET_DAY = 1015
    MWM_SET_DAY_AND_TOTAL = 201016
    MWM_SET_NIGHT = 1016
    MWM_SET_MANUAL = 1017
    DAY_STAGE = 1018
    NIGHT_STAGE = 1019
    PARTY_STAGE = 1020
    MANUAL_STAGE = 1021
    ROOM_TEMPERATURE_DAY_HK1_COOLING = 1022
    ROOM_TEMPERATURE_DAY_AND_TOTAL_HK1_COOLING = 201023
    ROOM_TEMPERATURE_NIGHT_HK1_COOLING = 1023
    ROOM_TEMPERATURE_DAY_HK2_COOLING = 1024
    ROOM_TEMPERATURE_DAY_AND_TOTAL_HK2_COOLING = 201025
    ROOM_TEMPERATURE_NIGHT_HK2_COOLING = 1025
    RESET = 1026
    RESTART_ISG = 1027


class LwzSystemStateRegisters(IsgRegisters):
    OPERATING_STATUS = 2001
    FAULT_STATUS = 2002
    BUS_STATUS = 2003
    DEFROST_INITIATED = 2004
    OPERATING_STATUS_2 = 2005


class LwzEnergyDataRegisters(IsgRegisters):
    HEAT_METER_HTG_DAY = 3001
    HEAT_METER_HTG_DAY_AND_TOTAL = 203002
    HEAT_METER_HTG_TTL_LOW = 3002
    HEAT_METER_HTG_TTL = 103002
    HEAT_METER_HTG_TTL_HI = 3003
    HEAT_METER_DHW_DAY = 3004
    HEAT_METER_DHW_DAY_AND_TOTAL = 203005
    HEAT_METER_DHW_TTL_LOW = 3005
    HEAT_METER_DHW_TTL = 103005
    HEAT_METER_DHW_TTL_HI = 3006
    HEAT_M_BOOST_HTG_TTL_LOW = 3007
    HEAT_M_BOOST_HTG_TTL = 103007
    HEAT_M_BOOST_HTG_TTL_HI = 3008
    HEAT_M_BOOST_DHW_TTL_LOW = 3009
    HEAT_M_BOOST_DHW_TTL = 103009
    HEAT_M_BOOST_DHW_HI = 3010
    HEAT_M_RECOVERY_DAY = 3011
    HEAT_M_RECOVERY_DAY_AND_TOTAL = 203012
    HEAT_M_RECOVERY_TTL_LOW = 3012
    HEAT_M_RECOVERY_TTL = 103012
    HEAT_M_RECOVERY_TTL_HI = 3013
    HM_SOLAR_HTG_DAY = 3014
    HM_SOLAR_HTG_DAY_AND_TOTAL = 203015
    HM_SOLAR_HTG_TOTAL_LOW = 3015
    HM_SOLAR_HTG_TOTAL = 103015
    HM_SOLAR_HTG_TOTAL_HI = 3016
    HM_SOLAR_DHW_DAY = 3017
    HM_SOLAR_DHW_DAY_AND_TOTAL = 203018
    HM_SOLAR_DWH_TOTAL_LOW = 3018
    HM_SOLAR_DWH_TOTAL = 103018
    HM_SOLAR_DWH_TOTAL_HI = 3019
    HM_COOLING_TOTAL_LOW = 3020
    HM_COOLING_TOTAL = 103020
    HM_COOLING_TOTAL_HI = 3021
    PWR_CON_HTG_DAY = 3022
    PWR_CON_HTG_DAY_AND_TOTAL = 203023
    PWR_CON_HTG_TTL_LOW = 3023
    PWR_CON_HTG_TTL = 103023
    PWR_CON_HTG_TTL_HI = 3024
    PWR_CON_DHW_DAY = 3025
    PWR_CON_DHW_DAY_AND_TOTAL = 203026
    PWR_CON_DHW_TTL_LOW = 3026
    PWR_CON_DHW_TTL = 103026
    PWR_CON_DHW_TTL_HI = 3027
    COMPRESSOR_HEATING = 3028
    COMPRESSOR_COOLING = 3029
    COMPRESSOR_DHW = 3030
    ELEC_BOOSTER_HEATING = 3031
    ELEC_BOOSTER_DHW = 3032


LWZ_SYSTEM_VALUES_REGISTERS = {
    LwzSystemValuesRegisters.ACTUAL_ROOM_T_HC1: ModbusRegister(address=1, name="ACTUAL ROOM T HC1", unit="°C", min=-20.0, max=60.0, data_type=2, key=LwzSystemValuesRegisters.ACTUAL_ROOM_T_HC1),
    LwzSystemValuesRegisters.SET_ROOM_TEMPERATURE_HC1: ModbusRegister(
        address=2, name="SET ROOM TEMPERATURE HC1", unit="°C", min=-20.0, max=60.0, data_type=2, key=LwzSystemValuesRegisters.SET_ROOM_TEMPERATURE_HC1
    ),
    LwzSystemValuesRegisters.RELATIVE_HUMIDITY_HC1: ModbusRegister(
        address=3, name="RELATIVE HUMIDITY HC1", unit="%", min=0.0, max=100.0, data_type=2, key=LwzSystemValuesRegisters.RELATIVE_HUMIDITY_HC1
    ),
    LwzSystemValuesRegisters.ACTUAL_ROOM_T_HC2: ModbusRegister(address=4, name="ACTUAL ROOM T HC2", unit="°C", min=-20.0, max=60.0, data_type=2, key=LwzSystemValuesRegisters.ACTUAL_ROOM_T_HC2),
    LwzSystemValuesRegisters.SET_ROOM_TEMPERATURE_HC2: ModbusRegister(
        address=5, name="SET ROOM TEMPERATURE HC2", unit="°C", min=-20.0, max=60.0, data_type=2, key=LwzSystemValuesRegisters.SET_ROOM_TEMPERATURE_HC2
    ),
    LwzSystemValuesRegisters.RELATIVE_HUMIDITY_HC2: ModbusRegister(
        address=6, name="RELATIVE HUMIDITY HC2", unit="%", min=0.0, max=100.0, data_type=2, key=LwzSystemValuesRegisters.RELATIVE_HUMIDITY_HC2
    ),
    LwzSystemValuesRegisters.OUTSIDE_TEMPERATURE: ModbusRegister(address=7, name="OUTSIDE TEMPERATURE", unit="°C", min=-60.0, max=80.0, data_type=2, key=LwzSystemValuesRegisters.OUTSIDE_TEMPERATURE),
    LwzSystemValuesRegisters.ACTUAL_VALUE_HC1: ModbusRegister(address=8, name="ACTUAL VALUE HC1", unit="°C", min=0.0, max=90.0, data_type=2, key=LwzSystemValuesRegisters.ACTUAL_VALUE_HC1),
    LwzSystemValuesRegisters.SET_VALUE_HC1: ModbusRegister(address=9, name="SET VALUE HC1", unit="°C", min=0.0, max=65.0, data_type=2, key=LwzSystemValuesRegisters.SET_VALUE_HC1),
    LwzSystemValuesRegisters.ACTUAL_VALUE_HC2: ModbusRegister(address=10, name="ACTUAL VALUE HC2", unit="°C", min=0.0, max=90.0, data_type=2, key=LwzSystemValuesRegisters.ACTUAL_VALUE_HC2),
    LwzSystemValuesRegisters.SET_VALUE_HC2: ModbusRegister(address=11, name="SET VALUE HC2", unit="°C", min=0.0, max=65.0, data_type=2, key=LwzSystemValuesRegisters.SET_VALUE_HC2),
    LwzSystemValuesRegisters.FLOW_TEMPERATURE: ModbusRegister(address=12, name="FLOW TEMPERATURE", unit="°C", min=0.0, max=90.0, data_type=2, key=LwzSystemValuesRegisters.FLOW_TEMPERATURE),
    LwzSystemValuesRegisters.RETURN_TEMPERATURE: ModbusRegister(address=13, name="RETURN TEMPERATURE", unit="°C", min=0.0, max=90.0, data_type=2, key=LwzSystemValuesRegisters.RETURN_TEMPERATURE),
    LwzSystemValuesRegisters.PRESSURE_HTG_CIRC: ModbusRegister(address=14, name="PRESSURE HTG CIRC", unit="bar", min=0.0, max=6.0, data_type=2, key=LwzSystemValuesRegisters.PRESSURE_HTG_CIRC),
    LwzSystemValuesRegisters.FLOW_RATE: ModbusRegister(address=15, name="FLOW RATE ", unit="l/min", min=None, max=None, data_type=2, key=LwzSystemValuesRegisters.FLOW_RATE),
    LwzSystemValuesRegisters.ACTUAL_DHW_T: ModbusRegister(address=16, name="ACTUAL DHW T", unit="°C", min=10.0, max=65.0, data_type=2, key=LwzSystemValuesRegisters.ACTUAL_DHW_T),
    LwzSystemValuesRegisters.DHW_SET_TEMPERATURE: ModbusRegister(address=17, name="DHW SET TEMPERATURE", unit="°C", min=10.0, max=65.0, data_type=2, key=LwzSystemValuesRegisters.DHW_SET_TEMPERATURE),
    LwzSystemValuesRegisters.VENTILATION_AIR_ACTUAL_FAN_SPEED: ModbusRegister(
        address=18, name="VENTILATION AIR ACTUAL FAN SPEED", unit="Hz", min=0.0, max=100.0, data_type=6, key=LwzSystemValuesRegisters.VENTILATION_AIR_ACTUAL_FAN_SPEED
    ),
    LwzSystemValuesRegisters.VENTILATION_AIR_SET_FLOW_RATE: ModbusRegister(
        address=19, name="VENTILATION AIR SET FLOW RATE", unit="m³/h", min=0.0, max=300.0, data_type=6, key=LwzSystemValuesRegisters.VENTILATION_AIR_SET_FLOW_RATE
    ),
    LwzSystemValuesRegisters.EXTRACT_AIR_ACTUAL_FAN_SPEED: ModbusRegister(
        address=20, name="EXTRACT AIR ACTUAL FAN SPEED", unit="Hz", min=0.0, max=100.0, data_type=6, key=LwzSystemValuesRegisters.EXTRACT_AIR_ACTUAL_FAN_SPEED
    ),
    LwzSystemValuesRegisters.EXTRACT_AIR_SET_FLOW_RATE: ModbusRegister(
        address=21, name="EXTRACT AIR SET FLOW RATE", unit="m³/h", min=0.0, max=300.0, data_type=6, key=LwzSystemValuesRegisters.EXTRACT_AIR_SET_FLOW_RATE
    ),
    LwzSystemValuesRegisters.EXTRACT_AIR_HUMIDITY: ModbusRegister(
        address=22, name="EXTRACT AIR HUMIDITY", unit="%", min=0.0, max=100.0, data_type=6, key=LwzSystemValuesRegisters.EXTRACT_AIR_HUMIDITY
    ),
    LwzSystemValuesRegisters.EXTRACT_AIR_TEMP: ModbusRegister(address=23, name="EXTRACT AIR TEMP", unit="°C", min=0.0, max=65535.0, data_type=2, key=LwzSystemValuesRegisters.EXTRACT_AIR_TEMP),
    LwzSystemValuesRegisters.EXTRACT_AIR_DEW_POINT: ModbusRegister(
        address=24, name="EXTRACT AIR DEW POINT", unit="°C", min=0.0, max=65535.0, data_type=2, key=LwzSystemValuesRegisters.EXTRACT_AIR_DEW_POINT
    ),
    LwzSystemValuesRegisters.DEW_POINT_TEMP_HC1: ModbusRegister(address=25, name="DEW POINT TEMP HC1", unit="°C", min=-40.0, max=30.0, data_type=2, key=LwzSystemValuesRegisters.DEW_POINT_TEMP_HC1),
    LwzSystemValuesRegisters.DEW_POINT_TEMP_HC2: ModbusRegister(address=26, name="DEW POINT TEMP HC2", unit="°C", min=-40.0, max=30.0, data_type=2, key=LwzSystemValuesRegisters.DEW_POINT_TEMP_HC2),
    LwzSystemValuesRegisters.COLLECTOR_TEMPERATURE: ModbusRegister(
        address=27, name="COLLECTOR TEMPERATURE", unit="°C", min=-60.0, max=200.0, data_type=2, key=LwzSystemValuesRegisters.COLLECTOR_TEMPERATURE
    ),
    LwzSystemValuesRegisters.HOT_GAS_TEMPERATURE: ModbusRegister(address=28, name="HOT GAS TEMPERATURE", unit="°C", min=0.0, max=140.0, data_type=2, key=LwzSystemValuesRegisters.HOT_GAS_TEMPERATURE),
    LwzSystemValuesRegisters.HIGH_PRESSURE: ModbusRegister(address=29, name="HIGH PRESSURE", unit="bar", min=0.0, max=50.0, data_type=7, key=LwzSystemValuesRegisters.HIGH_PRESSURE),
    LwzSystemValuesRegisters.LOW_PRESSURE: ModbusRegister(address=30, name="LOW PRESSURE", unit="bar", min=0.0, max=25.0, data_type=7, key=LwzSystemValuesRegisters.LOW_PRESSURE),
    LwzSystemValuesRegisters.COMPRESSOR_STARTS_HI: ModbusRegister(address=31, name="COMPRESSOR STARTS", unit="", min=0.0, max=65535.0, data_type=6, key=LwzSystemValuesRegisters.COMPRESSOR_STARTS_HI),
    LwzSystemValuesRegisters.COMPRESSOR_SPEED: ModbusRegister(address=32, name="COMPRESSOR SPEED", unit="Hz", min=0.0, max=240.0, data_type=2, key=LwzSystemValuesRegisters.COMPRESSOR_SPEED),
    LwzSystemValuesRegisters.MIXED_WATER_AMOUNT: ModbusRegister(address=33, name="MIXED WATER AMOUNT", unit="l", min=0.0, max=65535.0, data_type=6, key=LwzSystemValuesRegisters.MIXED_WATER_AMOUNT),
    LwzSystemValuesRegisters.COMPRESSOR_STARTS_LOW: ModbusRegister(
        address=34, name="COMPRESSOR STARTS", unit="", min=0.0, max=65535.0, data_type=6, key=LwzSystemValuesRegisters.COMPRESSOR_STARTS_LOW
    ),
}

LWZ_SYSTEM_PARAMETERS_REGISTERS = {
    LwzSystemParametersRegisters.OPERATING_MODE: ModbusRegister(address=1001, name="OPERATING MODE", unit="", min=0.0, max=14.0, data_type=8, key=LwzSystemParametersRegisters.OPERATING_MODE),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1: ModbusRegister(
        address=1002, name="ROOM TEMPERATURE DAY", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1
    ),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK1: ModbusRegister(
        address=1003, name="ROOM TEMPERATURE NIGHT", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK1
    ),
    LwzSystemParametersRegisters.MANUAL_HC_SET_HK1: ModbusRegister(address=1004, name="MANUAL HC SET", unit="°C", min=10.0, max=65.0, data_type=2, key=LwzSystemParametersRegisters.MANUAL_HC_SET_HK1),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK2: ModbusRegister(
        address=1005, name="ROOM TEMPERATURE DAY", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK2
    ),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK2: ModbusRegister(
        address=1006, name="ROOM TEMPERATURE NIGHT", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK2
    ),
    LwzSystemParametersRegisters.MANUAL_HC_SET_HK2: ModbusRegister(address=1007, name="MANUAL HC SET", unit="°C", min=10.0, max=65.0, data_type=2, key=LwzSystemParametersRegisters.MANUAL_HC_SET_HK2),
    LwzSystemParametersRegisters.GRADIENT_HK1: ModbusRegister(address=1008, name="GRADIENT", unit="", min=0.0, max=5.0, data_type=7, key=LwzSystemParametersRegisters.GRADIENT_HK1),
    LwzSystemParametersRegisters.LOW_END_HK1: ModbusRegister(address=1009, name="LOW END", unit="°C", min=0.0, max=20.0, data_type=2, key=LwzSystemParametersRegisters.LOW_END_HK1),
    LwzSystemParametersRegisters.GRADIENT_HK2: ModbusRegister(address=1010, name="GRADIENT", unit="", min=0.0, max=5.0, data_type=7, key=LwzSystemParametersRegisters.GRADIENT_HK2),
    LwzSystemParametersRegisters.LOW_END_HK2: ModbusRegister(address=1011, name="LOW END", unit="°C", min=0.0, max=20.0, data_type=2, key=LwzSystemParametersRegisters.LOW_END_HK2),
    LwzSystemParametersRegisters.DHW_SET_DAY: ModbusRegister(address=1012, name="DHW SET DAY", unit="°C", min=10.0, max=55.0, data_type=2, key=LwzSystemParametersRegisters.DHW_SET_DAY),
    LwzSystemParametersRegisters.DHW_SET_NIGHT: ModbusRegister(address=1013, name="DHW SET NIGHT", unit="°C", min=10.0, max=55.0, data_type=2, key=LwzSystemParametersRegisters.DHW_SET_NIGHT),
    LwzSystemParametersRegisters.DHW_SET_MANUAL: ModbusRegister(address=1014, name="DHW SET MANUAL", unit="°C", min=10.0, max=65.0, data_type=2, key=LwzSystemParametersRegisters.DHW_SET_MANUAL),
    LwzSystemParametersRegisters.MWM_SET_DAY: ModbusRegister(address=1015, name="MWM SET DAY", unit="l", min=50.0, max=288.0, data_type=6, key=LwzSystemParametersRegisters.MWM_SET_DAY),
    LwzSystemParametersRegisters.MWM_SET_NIGHT: ModbusRegister(address=1016, name="MWM SET NIGHT", unit="l", min=50.0, max=288.0, data_type=6, key=LwzSystemParametersRegisters.MWM_SET_NIGHT),
    LwzSystemParametersRegisters.MWM_SET_MANUAL: ModbusRegister(address=1017, name="MWM SET MANUAL", unit="l", min=50.0, max=288.0, data_type=6, key=LwzSystemParametersRegisters.MWM_SET_MANUAL),
    LwzSystemParametersRegisters.DAY_STAGE: ModbusRegister(address=1018, name="DAY STAGE", unit="", min=0.0, max=3.0, data_type=6, key=LwzSystemParametersRegisters.DAY_STAGE),
    LwzSystemParametersRegisters.NIGHT_STAGE: ModbusRegister(address=1019, name="NIGHT STAGE", unit="", min=0.0, max=3.0, data_type=6, key=LwzSystemParametersRegisters.NIGHT_STAGE),
    LwzSystemParametersRegisters.PARTY_STAGE: ModbusRegister(address=1020, name="PARTY STAGE", unit="", min=0.0, max=3.0, data_type=6, key=LwzSystemParametersRegisters.PARTY_STAGE),
    LwzSystemParametersRegisters.MANUAL_STAGE: ModbusRegister(address=1021, name="MANUAL STAGE", unit="", min=0.0, max=3.0, data_type=6, key=LwzSystemParametersRegisters.MANUAL_STAGE),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1_COOLING: ModbusRegister(
        address=1022, name="ROOM TEMPERATURE DAY", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1_COOLING
    ),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK1_COOLING: ModbusRegister(
        address=1023, name="ROOM TEMPERATURE NIGHT", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK1_COOLING
    ),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK2_COOLING: ModbusRegister(
        address=1024, name="ROOM TEMPERATURE DAY", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK2_COOLING
    ),
    LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK2_COOLING: ModbusRegister(
        address=1025, name="ROOM TEMPERATURE NIGHT", unit="°C", min=10.0, max=30.0, data_type=2, key=LwzSystemParametersRegisters.ROOM_TEMPERATURE_NIGHT_HK2_COOLING
    ),
    LwzSystemParametersRegisters.RESET: ModbusRegister(address=1026, name="RESET", unit="", min=0.0, max=1.0, data_type=6, key=LwzSystemParametersRegisters.RESET),
    LwzSystemParametersRegisters.RESTART_ISG: ModbusRegister(address=1027, name="RESTART ISG", unit="", min=0.0, max=2.0, data_type=6, key=LwzSystemParametersRegisters.RESTART_ISG),
}

LWZ_SYSTEM_STATE_REGISTERS = {
    LwzSystemStateRegisters.OPERATING_STATUS: ModbusRegister(address=2001, name="OPERATING STATUS", unit="", min=0.0, max=65535.0, data_type=6, key=LwzSystemStateRegisters.OPERATING_STATUS),
    LwzSystemStateRegisters.FAULT_STATUS: ModbusRegister(address=2002, name="FAULT STATUS", unit="", min=0.0, max=1.0, data_type=6, key=LwzSystemStateRegisters.FAULT_STATUS),
    LwzSystemStateRegisters.BUS_STATUS: ModbusRegister(address=2003, name="BUS STATUS", unit="", min=-4.0, max=0.0, data_type=6, key=LwzSystemStateRegisters.BUS_STATUS),
    LwzSystemStateRegisters.DEFROST_INITIATED: ModbusRegister(address=2004, name="DEFROST INITIATED", unit="", min=0.0, max=1.0, data_type=6, key=LwzSystemStateRegisters.DEFROST_INITIATED),
    LwzSystemStateRegisters.OPERATING_STATUS_2: ModbusRegister(address=2005, name="OPERATING STATUS 2", unit="", min=0.0, max=65535.0, data_type=6, key=LwzSystemStateRegisters.OPERATING_STATUS_2),
}

LWZ_ENERGY_DATA_REGISTERS = {
    LwzEnergyDataRegisters.HEAT_METER_HTG_DAY: ModbusRegister(address=3001, name="HEAT METER HTG DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_HTG_DAY),
    LwzEnergyDataRegisters.HEAT_METER_HTG_TTL_LOW: ModbusRegister(
        address=3002, name="HEAT METER HTG TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_HTG_TTL_LOW
    ),
    LwzEnergyDataRegisters.HEAT_METER_HTG_TTL_HI: ModbusRegister(
        address=3003, name="HEAT METER HTG TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_HTG_TTL_HI
    ),
    LwzEnergyDataRegisters.HEAT_METER_DHW_DAY: ModbusRegister(address=3004, name="HEAT METER DHW DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_DHW_DAY),
    LwzEnergyDataRegisters.HEAT_METER_DHW_TTL_LOW: ModbusRegister(
        address=3005, name="HEAT METER DHW TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_DHW_TTL_LOW
    ),
    LwzEnergyDataRegisters.HEAT_METER_DHW_TTL_HI: ModbusRegister(
        address=3006, name="HEAT METER DHW TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_METER_DHW_TTL_HI
    ),
    LwzEnergyDataRegisters.HEAT_M_BOOST_HTG_TTL_LOW: ModbusRegister(
        address=3007, name="HEAT M BOOST HTG TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_BOOST_HTG_TTL_LOW
    ),
    LwzEnergyDataRegisters.HEAT_M_BOOST_HTG_TTL_HI: ModbusRegister(
        address=3008, name="HEAT M BOOST HTG TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_BOOST_HTG_TTL_HI
    ),
    LwzEnergyDataRegisters.HEAT_M_BOOST_DHW_TTL_LOW: ModbusRegister(
        address=3009, name="HEAT M BOOST DHW TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_BOOST_DHW_TTL_LOW
    ),
    LwzEnergyDataRegisters.HEAT_M_BOOST_DHW_HI: ModbusRegister(address=3010, name="HEAT M BOOST DHW", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_BOOST_DHW_HI),
    LwzEnergyDataRegisters.HEAT_M_RECOVERY_DAY: ModbusRegister(address=3011, name="HEAT M RECOVERY DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_RECOVERY_DAY),
    LwzEnergyDataRegisters.HEAT_M_RECOVERY_TTL_LOW: ModbusRegister(
        address=3012, name="HEAT M RECOVERY TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_RECOVERY_TTL_LOW
    ),
    LwzEnergyDataRegisters.HEAT_M_RECOVERY_TTL_HI: ModbusRegister(
        address=3013, name="HEAT M RECOVERY TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HEAT_M_RECOVERY_TTL_HI
    ),
    LwzEnergyDataRegisters.HM_SOLAR_HTG_DAY: ModbusRegister(address=3014, name="HM SOLAR HTG DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_HTG_DAY),
    LwzEnergyDataRegisters.HM_SOLAR_HTG_TOTAL_LOW: ModbusRegister(
        address=3015, name="HM SOLAR HTG TOTAL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_HTG_TOTAL_LOW
    ),
    LwzEnergyDataRegisters.HM_SOLAR_HTG_TOTAL_HI: ModbusRegister(
        address=3016, name="HM SOLAR HTG TOTAL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_HTG_TOTAL_HI
    ),
    LwzEnergyDataRegisters.HM_SOLAR_DHW_DAY: ModbusRegister(address=3017, name="HM SOLAR DHW DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_DHW_DAY),
    LwzEnergyDataRegisters.HM_SOLAR_DWH_TOTAL_LOW: ModbusRegister(
        address=3018, name="HM SOLAR DWH TOTAL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_DWH_TOTAL_LOW
    ),
    LwzEnergyDataRegisters.HM_SOLAR_DWH_TOTAL_HI: ModbusRegister(
        address=3019, name="HM SOLAR DWH TOTAL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HM_SOLAR_DWH_TOTAL_HI
    ),
    LwzEnergyDataRegisters.HM_COOLING_TOTAL_LOW: ModbusRegister(address=3020, name="HM COOLING TOTAL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.HM_COOLING_TOTAL_LOW),
    LwzEnergyDataRegisters.HM_COOLING_TOTAL_HI: ModbusRegister(address=3021, name="HM COOLING TOTAL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.HM_COOLING_TOTAL_HI),
    LwzEnergyDataRegisters.PWR_CON_HTG_DAY: ModbusRegister(address=3022, name="PWR CON HTG DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_HTG_DAY),
    LwzEnergyDataRegisters.PWR_CON_HTG_TTL_LOW: ModbusRegister(address=3023, name="PWR CON HTG TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_HTG_TTL_LOW),
    LwzEnergyDataRegisters.PWR_CON_HTG_TTL_HI: ModbusRegister(address=3024, name="PWR CON HTG TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_HTG_TTL_HI),
    LwzEnergyDataRegisters.PWR_CON_DHW_DAY: ModbusRegister(address=3025, name="PWR CON DHW DAY", unit="kWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_DHW_DAY),
    LwzEnergyDataRegisters.PWR_CON_DHW_TTL_LOW: ModbusRegister(address=3026, name="PWR CON DHW TTL", unit="kWh", min=0.0, max=999.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_DHW_TTL_LOW),
    LwzEnergyDataRegisters.PWR_CON_DHW_TTL_HI: ModbusRegister(address=3027, name="PWR CON DHW TTL", unit="MWh", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.PWR_CON_DHW_TTL_HI),
    LwzEnergyDataRegisters.COMPRESSOR_HEATING: ModbusRegister(address=3028, name="COMPRESSOR HEATING", unit="h", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.COMPRESSOR_HEATING),
    LwzEnergyDataRegisters.COMPRESSOR_COOLING: ModbusRegister(address=3029, name="COMPRESSOR COOLING", unit="h", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.COMPRESSOR_COOLING),
    LwzEnergyDataRegisters.COMPRESSOR_DHW: ModbusRegister(address=3030, name="COMPRESSOR DHW", unit="h", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.COMPRESSOR_DHW),
    LwzEnergyDataRegisters.ELEC_BOOSTER_HEATING: ModbusRegister(
        address=3031, name="ELEC BOOSTER HEATING", unit="h", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.ELEC_BOOSTER_HEATING
    ),
    LwzEnergyDataRegisters.ELEC_BOOSTER_DHW: ModbusRegister(address=3032, name="ELEC BOOSTER DHW", unit="h", min=0.0, max=65535.0, data_type=6, key=LwzEnergyDataRegisters.ELEC_BOOSTER_DHW),
}


class OperatingMode(Enum):
    """Enum for the operation mode of the heat pump."""

    # AUTOMATIK
    AUTOMATIC = 11
    # BEREITSCHAFT
    STANDBY = 1
    # TAGBETRIEB
    DAY_MODE = 3
    # ABSENKBETRIEB
    SETBACK_MODE = 4
    # WARMWASSER
    DHW = 5
    # HANDBETRIEB
    MANUAL_MODE = 14
    # NOTBETRIEB
    EMERGENCY_OPERATION = 0


class LwzStiebelEltronAPI(StiebelEltronAPI):
    def __init__(self, host: str, port: int = 502, device_id: int = 1) -> None:
        super().__init__(
            [
                ModbusRegisterBlock(base_address=0, count=34, name="System Values", registers=LWZ_SYSTEM_VALUES_REGISTERS, register_type=RegisterType.INPUT_REGISTER),
                ModbusRegisterBlock(base_address=1000, count=27, name="System Parameters", registers=LWZ_SYSTEM_PARAMETERS_REGISTERS, register_type=RegisterType.HOLDING_REGISTER),
                ModbusRegisterBlock(base_address=2000, count=5, name="System State", registers=LWZ_SYSTEM_STATE_REGISTERS, register_type=RegisterType.INPUT_REGISTER),
                ModbusRegisterBlock(base_address=3000, count=32, name="Energy Data", registers=LWZ_ENERGY_DATA_REGISTERS, register_type=RegisterType.INPUT_REGISTER),
                ModbusRegisterBlock(base_address=4000, count=3, name="Energy Management Settings", registers=ENERGY_MANAGEMENT_SETTINGS_REGISTERS, register_type=RegisterType.HOLDING_REGISTER),
                ModbusRegisterBlock(base_address=5000, count=2, name="Energy System Information", registers=ENERGY_SYSTEM_INFORMATION_REGISTERS, register_type=RegisterType.INPUT_REGISTER),
            ],
            host,
            port,
            device_id,
        )

    async def async_update(self):
        """Request current values from heat pump."""
        await super().async_update()
        for registerblock in self._register_blocks:
            if registerblock.name == ENERGY_DATA_BLOCK_NAME:
                registers = [r.value for r in LwzEnergyDataRegisters]
                registers.sort()
                for register in registers:
                    if register > VIRTUAL_REGISTER_OFFSET:
                        if register > VIRTUAL_TOTAL_AND_DAY_REGISTER_OFFSET:
                            total_key = LwzEnergyDataRegisters(register - VIRTUAL_TOTAL_AND_DAY_REGISTER_OFFSET + VIRTUAL_REGISTER_OFFSET)
                            day_key = LwzEnergyDataRegisters(register - VIRTUAL_TOTAL_AND_DAY_REGISTER_OFFSET - 1)
                            total_value = self._data.get(total_key)
                            day_value = self._data.get(day_key)
                            if total_value is not None and day_value is not None:
                                prev_value = self._previous_data.get(LwzEnergyDataRegisters(register))
                                if prev_value is not None:
                                    self._data[LwzEnergyDataRegisters(register)] = max(total_value + day_value, prev_value)
                                else:
                                    self._data[LwzEnergyDataRegisters(register)] = total_value + day_value
                        else:
                            low_key = LwzEnergyDataRegisters(register - VIRTUAL_REGISTER_OFFSET)
                            high_key = LwzEnergyDataRegisters(register - VIRTUAL_REGISTER_OFFSET + 1)
                            high_value = self._data.get(high_key)
                            low_value = self._data.get(low_key)
                            if high_value is not None and low_value is not None:
                                self._data[LwzEnergyDataRegisters(register)] = high_value * 1000 + low_value

        compressor_starts_high = self.get_register_value(LwzSystemValuesRegisters.COMPRESSOR_STARTS_HI)
        if compressor_starts_high is None or compressor_starts_high == 32768:
            self._data[LwzSystemValuesRegisters.COMPRESSOR_STARTS] = 32768
        else:
            compressor_starts_low = self.get_register_value(LwzSystemValuesRegisters.COMPRESSOR_STARTS_LOW)
            if compressor_starts_low is None:
                self._data[LwzSystemValuesRegisters.COMPRESSOR_STARTS] = compressor_starts_high * 1000
            else:
                self._data[LwzSystemValuesRegisters.COMPRESSOR_STARTS] = compressor_starts_high * 1000 + compressor_starts_low

    def get_current_temp(self):
        """Get the current room temperature."""
        return self.get_register_value(LwzSystemValuesRegisters.ACTUAL_ROOM_T_HC1)

    def get_target_temp(self):
        """Get the target room temperature."""
        return self.get_register_value(LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1)

    async def set_target_temp(self, temp):
        """Set the target room temperature (day)(HC1)."""
        await self.write_register_value(LwzSystemParametersRegisters.ROOM_TEMPERATURE_DAY_HK1, temp)

    def get_current_humidity(self):
        """Get the current room humidity."""
        return self.get_register_value(LwzSystemValuesRegisters.RELATIVE_HUMIDITY_HC1)

    # Handle operation mode

    def get_operation(self) -> OperatingMode:
        """Return the current mode of operation."""
        op_mode = int(self.get_register_value(LwzSystemParametersRegisters.OPERATING_MODE))
        return OperatingMode(op_mode)

    async def set_operation(self, mode: OperatingMode):
        """Set the operation mode."""
        await self.write_register_value(LwzSystemParametersRegisters.OPERATING_MODE, mode.value)

    def get_heating_status(self) -> bool:
        """Return heater status."""
        return bool(int(self.get_register_value(LwzSystemStateRegisters.OPERATING_STATUS)) & (1 << 2))

    def get_cooling_status(self) -> bool:
        """Cooling status."""
        return bool(int(self.get_register_value(LwzSystemStateRegisters.OPERATING_STATUS)) & (1 << 3))

    def get_filter_alarm_status(self) -> bool:
        """Return filter alarm."""
        filter_mask = (1 << 8) | (1 << 12) | (1 << 13)
        return bool(int(self.get_register_value(LwzSystemStateRegisters.OPERATING_STATUS)) & filter_mask)



"""
Connection to a Stiebel Eltron ModBus API.

See API details:
https://www.stiebel-eltron.de/content/dam/ste/de/de/home/services/Downloadlisten/ISG%20Modbus_Stiebel_Bedienungsanleitung.pdf

Types of data:

Data | Value      | Multiplier  | Multiplier  | Signed | Step   | Step
type | range      | for reading | for writing |        | size 1 | size 5
-----|------------|-------------|-------------|--------|--------|-------
2    | -3276.8 to | 0.1         | 10          | Yes    | 0.1    | 0.5
     |  3276.7    |             |             |        |        |
6    | 0 to 65535 | 1           | 1           | No     | 1      | 5
7    | -327.68 to | 0.01        | 100         | Yes    | 0.01   | 0.05
     |  327.67    |             |             |        |        |
8    | 0 to 255   | 1           | 1           | No     | 1      | 5
"""

from pymodbus.client.mixin import ModbusClientMixin

# Error - sensor lead is missing or disconnected.
ERROR_NOTAVAILABLE = -60
# Error - short circuit of the sensor lead.
ERROR_SHORTCUT = -50
# Error - object unavailable.
ERROR_OBJ_UNAVAILBLE = 0x8000

UNAVAILABLE_OBJECT = 32768

# Block 1 System values (Read input register) - page 29
B1_START_ADDR = 0

B1_REGMAP_INPUT = {
    # HC = Heating Circuit
    "ACTUAL_ROOM_TEMPERATURE_HC1": {"addr": 0, "type": 2, "value": 0},
    "SET_ROOM_TEMPERATURE_HC1": {"addr": 1, "type": 2, "value": 0},
    "RELATIVE_HUMIDITY_HC1": {"addr": 2, "type": 2, "value": 0},
    "ACTUAL_ROOM_TEMPERATURE_HC2": {"addr": 3, "type": 2, "value": 0},
    "SET_ROOM_TEMPERATURE_HC2": {"addr": 4, "type": 2, "value": 0},
    "RELATIVE_HUMIDITY_HC2": {"addr": 5, "type": 2, "value": 0},
    "OUTSIDE_TEMPERATURE": {"addr": 6, "type": 2, "value": 0},
    "ACTUAL_VALUE_HC1": {"addr": 7, "type": 2, "value": 0},
    "SET_VALUE_HC1": {"addr": 8, "type": 2, "value": 0},
    "ACTUAL_VALUE_HC2": {"addr": 9, "type": 2, "value": 0},
    "SET_VALUE_HC2": {"addr": 10, "type": 2, "value": 0},
    "FLOW_TEMPERATURE": {"addr": 11, "type": 2, "value": 0},
    "RETURN_TEMPERATURE": {"addr": 12, "type": 2, "value": 0},
    "PRESSURE_HEATING_CIRCUIT": {"addr": 13, "type": 2, "value": 0},
    "FLOW_RATE": {"addr": 14, "type": 2, "value": 0},
    "ACTUAL_DHW_TEMPERATURE": {"addr": 15, "type": 2, "value": 0},
    "SET_DHW_TEMPERATURE": {"addr": 16, "type": 2, "value": 0},
    "VENTILATION_AIR_ACTUAL_FAN_SPEED": {"addr": 17, "type": 6, "value": 0},
    "VENTILATION_AIR_SET_FLOW_RATE": {"addr": 18, "type": 6, "value": 0},
    "EXTRACT_AIR_ACTUAL_FAN_SPEED": {"addr": 19, "type": 6, "value": 0},
    "EXTRACT_AIR_SET_FLOW_RATE": {"addr": 20, "type": 6, "value": 0},
    "EXTRACT_AIR_HUMIDITY": {"addr": 21, "type": 6, "value": 0},
    "EXTRACT_AIR_TEMPERATURE": {"addr": 22, "type": 2, "value": 0},
    "EXTRACT_AIR_DEW_POINT": {"addr": 23, "type": 2, "value": 0},
    "DEW_POINT_TEMPERATUR_HC1": {"addr": 24, "type": 2, "value": 0},
    "DEW_POINT_TEMPERATUR_HC2": {"addr": 25, "type": 2, "value": 0},
    "COLLECTOR_TEMPERATURE": {"addr": 26, "type": 2, "value": 0},
    "HOT_GAS_TEMPERATURE": {"addr": 27, "type": 2, "value": 0},
    "HIGH_PRESSURE": {"addr": 28, "type": 7, "value": 0},
    "LOW_PRESSURE": {"addr": 29, "type": 7, "value": 0},
    "COMPRESSOR_STARTS": {"addr": 30, "type": 6, "value": 0},
    "COMPRESSOR_SPEED": {"addr": 31, "type": 2, "value": 0},
    "MIXED_WATER_AMOUNT": {"addr": 32, "type": 6, "value": 0},
}


# Block 2 System parameters (Read/write holding register) - page 30
B2_START_ADDR = 1000

B2_REGMAP_HOLDING = {
    "OPERATING_MODE": {"addr": 1000, "type": 8, "value": 0},
    "ROOM_TEMP_HEAT_DAY_HC1": {"addr": 1001, "type": 2, "value": 0},
    "ROOM_TEMP_HEAT_NIGHT_HC1": {"addr": 1002, "type": 2, "value": 0},
    "MANUAL_SET_TEMP_HC1": {"addr": 1003, "type": 2, "value": 0},
    "ROOM_TEMP_HEAT_DAY_HC2": {"addr": 1004, "type": 2, "value": 0},
    "ROOM_TEMP_HEAT_NIGHT_HC2": {"addr": 1005, "type": 2, "value": 0},
    "MANUAL_SET_TEAMP_HC2": {"addr": 1006, "type": 2, "value": 0},
    "GRADIENT_HC1": {"addr": 1007, "type": 7, "value": 0},
    "LOW_END_HC1": {"addr": 1008, "type": 2, "value": 0},
    "GRADIENT_HC2": {"addr": 1009, "type": 7, "value": 0},
    "LOW_END_HC2": {"addr": 1010, "type": 2, "value": 0},
    "DHW_TEMP_SET_DAY": {"addr": 1011, "type": 2, "value": 0},
    "DHW_TEMP_SET_NIGHT": {"addr": 1012, "type": 2, "value": 0},
    "DHW_TEMP_SET_MANUAL": {"addr": 1013, "type": 2, "value": 0},
    "MWM_SET_DAY": {"addr": 1014, "type": 6, "value": 0},
    "MWM_SET_NIGHT": {"addr": 1015, "type": 6, "value": 0},
    "MWM_SET_MANUAL": {"addr": 1016, "type": 6, "value": 0},
    "DAY_STAGE": {"addr": 1017, "type": 6, "value": 0},
    "NIGHT_STAGE": {"addr": 1018, "type": 6, "value": 0},
    "PARTY_STAGE": {"addr": 1019, "type": 6, "value": 0},
    "MANUAL_STAGE": {"addr": 1020, "type": 6, "value": 0},
    "ROOM_TEMP_COOL_DAY_HC1": {"addr": 1021, "type": 2, "value": 0},
    "ROOM_TEMP_COOL_NIGHT_HC1": {"addr": 1022, "type": 2, "value": 0},
    "ROOM_TEMP_COOL_DAY_HC2": {"addr": 1023, "type": 2, "value": 0},
    "ROOM_TEMP_COOL_NIGHT_HC2": {"addr": 1024, "type": 2, "value": 0},
    "RESET": {"addr": 1025, "type": 6, "value": 0},
    "RESTART_ISG": {"addr": 1026, "type": 6, "value": 0},
}

B2_OPERATING_MODE_READ = {
    # AUTOMATIK
    11: "AUTOMATIC",
    # BEREITSCHAFT
    1: "STANDBY",
    # TAGBETRIEB
    3: "DAY MODE",
    # ABSENKBETRIEB
    4: "SETBACK MODE",
    # WARMWASSER
    5: "DHW",
    # HANDBETRIEB
    14: "MANUAL MODE",
    # NOTBETRIEB
    0: "EMERGENCY OPERATION",
}

B2_OPERATING_MODE_WRITE = {value: key for key, value in B2_OPERATING_MODE_READ.items()}

B2_RESET = {"OFF": 0, "ON": 1}

B2_RESTART_ISG = {"OFF": 0, "RESET": 1, "MENU": 2}

# Block 3 System status (Read input register) - page 31
B3_START_ADDR = 2000

B3_REGMAP_INPUT = {"OPERATING_STATUS": {"addr": 2000, "type": 6, "value": 0}, "FAULT_STATUS": {"addr": 2001, "type": 6, "value": 0}, "BUS_STATUS": {"addr": 2002, "type": 6, "value": 0}}

B3_OPERATING_STATUS = {
    "SWITCHING_PROGRAM_ENABLED": (1 << 0),
    "COMPRESSOR": (1 << 1),
    "HEATING": (1 << 2),
    "COOLING": (1 << 3),
    "DHW": (1 << 4),
    "ELECTRIC_REHEATING": (1 << 5),
    "SERVICE": (1 << 6),
    "POWER-OFF": (1 << 7),
    "FILTER": (1 << 8),
    "VENTILATION": (1 << 9),
    "HEATING_CIRCUIT_PUMP": (1 << 10),
    "EVAPORATOR_DEFROST": (1 << 11),
    "FILTER_EXTRACT_AIR": (1 << 12),
    "FILTER_VENTILATION_AIR": (1 << 13),
    "HEAT-UP_PROGRAM": (1 << 14),
}

B3_FAULT_STATUS = {"NO_FAULT": 0, "FAULT": 1}

B3_BUS_STATUS = {"STATUS OK": 0, "STATUS ERROR": -1, "ERROR-PASSIVE": -2, "BUS-OFF": -3, "PHYSICAL-ERROR": -4}


class StiebelEltronAPI:
    """Stiebel Eltron API."""

    def __init__(self, conn: ModbusClientMixin, device_id=1, update_on_read=False):
        """Initialize Stiebel Eltron communication."""
        self._conn = conn
        self._block_1_input_regs = B1_REGMAP_INPUT
        self._block_2_holding_regs = B2_REGMAP_HOLDING
        self._block_3_input_regs = B3_REGMAP_INPUT
        self._device_id = device_id
        self._update_on_read = update_on_read

    def update(self):
        """Request current values from heat pump."""
        ret = True
        try:
            block_1_result_input = self._conn.read_input_registers(device_id=self._device_id, address=B1_START_ADDR, count=len(self._block_1_input_regs)).registers
            block_2_result_holding = self._conn.read_holding_registers(device_id=self._device_id, address=B2_START_ADDR, count=len(self._block_2_holding_regs)).registers
            block_3_result_input = self._conn.read_input_registers(device_id=self._device_id, address=B3_START_ADDR, count=len(self._block_3_input_regs)).registers

        except AttributeError:
            # The unit does not reply reliably
            ret = False
            print("Modbus read failed")
        else:
            for v in self._block_1_input_regs.values():
                v["value"] = block_1_result_input[v["addr"] - B1_START_ADDR]

            for v in self._block_2_holding_regs.values():
                v["value"] = block_2_result_holding[v["addr"] - B2_START_ADDR]

            for v in self._block_3_input_regs.values():
                v["value"] = block_3_result_input[v["addr"] - B3_START_ADDR]

        return ret

    def get_conv_val(self, name: str):
        """Read and convert value.

        Args:
            name: Name of value to be read.

        Returns:
            Actual value or None.
        """
        value_entry = self._block_1_input_regs.get(name)
        if value_entry is None:
            value_entry = self._block_2_holding_regs.get(name)
        if value_entry is None:
            value_entry = self._block_3_input_regs.get(name)
        if value_entry is None:
            return None

        if value_entry["type"] == 2:
            return value_entry["value"] * 0.1
        if value_entry["type"] == 7:
            return value_entry["value"] * 0.01

        return value_entry["value"]

    #    def get_raw_input_register(self, name):
    #        """Get raw register value by name."""
    #        if self._update_on_read:
    #            self.update()
    #        return self._block_1_input_regs[name]

    #    def get_raw_holding_register(self, name):
    #        """Get raw register value by name."""
    #        if self._update_on_read:
    #            self.update()
    #        return self._block_2_holding_regs[name]

    #    def set_raw_holding_register(self, name, value):
    #        """Write to register by name."""
    #        self._conn.write_register(
    #            device_id=self._device_id,
    #            address=(self._holding_regs[name]['addr']),
    #            value=value)

    # Handle room temperature & humidity

    def get_current_temp(self):
        """Get the current room temperature."""
        if self._update_on_read:
            self.update()
        return self.get_conv_val("ACTUAL_ROOM_TEMPERATURE_HC1")

    def get_target_temp(self):
        """Get the target room temperature."""
        if self._update_on_read:
            self.update()
        return self.get_conv_val("ROOM_TEMP_HEAT_DAY_HC1")

    def set_target_temp(self, temp: float):
        """Set the target room temperature (day)(HC1)."""
        self._conn.write_register(device_id=self._device_id, address=(self._block_2_holding_regs["ROOM_TEMP_HEAT_DAY_HC1"]["addr"]), value=round(temp * 10.0))

    def get_current_humidity(self):
        """Get the current room humidity."""
        if self._update_on_read:
            self.update()
        return self.get_conv_val("RELATIVE_HUMIDITY_HC1")

    # Handle operation mode

    def get_operation(self):
        """Return the current mode of operation."""
        if self._update_on_read:
            self.update()

        op_mode = self.get_conv_val("OPERATING_MODE")
        return B2_OPERATING_MODE_READ.get(op_mode, "UNKNOWN")

    def set_operation(self, mode: str):
        """Set the operation mode."""
        self._conn.write_register(device_id=self._device_id, address=(self._block_2_holding_regs["OPERATING_MODE"]["addr"]), value=B2_OPERATING_MODE_WRITE.get(mode))

    # Handle device status

    def get_heating_status(self):
        """Return heater status."""
        if self._update_on_read:
            self.update()
        return bool(self.get_conv_val("OPERATING_STATUS") & B3_OPERATING_STATUS["HEATING"])

    def get_cooling_status(self):
        """Cooling status."""
        if self._update_on_read:
            self.update()
        return bool(self.get_conv_val("OPERATING_STATUS") & B3_OPERATING_STATUS["COOLING"])

    def get_filter_alarm_status(self):
        """Return filter alarm."""
        if self._update_on_read:
            self.update()

        filter_mask = B3_OPERATING_STATUS["FILTER"] | B3_OPERATING_STATUS["FILTER_EXTRACT_AIR"] | B3_OPERATING_STATUS["FILTER_VENTILATION_AIR"]
        return bool(self.get_conv_val("OPERATING_STATUS") & filter_mask)
